//--------------------------------------------------------------------------------------
// File: main.fx
//
// The effect file for the main sample.  
// 
// Copyright (c) 2004 IMAGIRE Takashi. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

float    g_fInvTexSize;

//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
texture Texture;
sampler Sampler = sampler_state
{
    Texture = <Texture>;
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
//-----------------------------------------------------------------------------
texture TextureInfo;
sampler SamplerInfo = sampler_state
{
    Texture = <TextureInfo>;
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};


//=============================================================================
// ΐƂ
//=============================================================================
//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct LogVS_OUTPUT
{
    float4 Position : POSITION;
    float2 TexCoord0 : TEXCOORD0;
    float2 TexCoord1 : TEXCOORD1;
    float2 TexCoord2 : TEXCOORD2;
    float2 TexCoord3 : TEXCOORD3;
};


//-----------------------------------------------------------------------------
// Name: LogVS
//-----------------------------------------------------------------------------
LogVS_OUTPUT LogVS( float4 vPos : POSITION, 
                    float2 vTexCoord0 : TEXCOORD0 )
{
    LogVS_OUTPUT Output;
  
    // tranform vertex position into screen space
    Output.Position = vPos;
    
    // Just copy the texture coordinate through
    Output.TexCoord0 = vTexCoord0 + float2( 0.5*g_fInvTexSize, 0.5*g_fInvTexSize );
    Output.TexCoord1 = vTexCoord0 + float2( 1.5*g_fInvTexSize, 0.5*g_fInvTexSize );
    Output.TexCoord2 = vTexCoord0 + float2( 0.5*g_fInvTexSize, 1.5*g_fInvTexSize );
    Output.TexCoord3 = vTexCoord0 + float2( 1.5*g_fInvTexSize, 1.5*g_fInvTexSize );
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: LogPS
//-----------------------------------------------------------------------------
float4 LogPS( LogVS_OUTPUT In ) : COLOR
{
	const float3 RGB2Y   = float3( +0.29900f, +0.58700f, +0.11400f );
	const float EPSILON = 0.00001;
	float4 output;
	
    float l0 = dot( RGB2Y, tex2D( Sampler, In.TexCoord0 ));
    float l1 = dot( RGB2Y, tex2D( Sampler, In.TexCoord1 ));
    float l2 = dot( RGB2Y, tex2D( Sampler, In.TexCoord2 ));
    float l3 = dot( RGB2Y, tex2D( Sampler, In.TexCoord3 ));
    
    float l_max = max(max(l0,l1),max(l2,l3));
    
    float total =  log( EPSILON + l0 )
				 + log( EPSILON + l1 )
				 + log( EPSILON + l2 )
				 + log( EPSILON + l3 );
				 
	output.r = l_max;
	output.gba = 0.25f * total;
	
	return output;
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique LogTechnique
{
    pass P0
    {
        VertexShader = compile vs_1_1 LogVS();
        PixelShader  = compile ps_2_0 LogPS();
    }
}





//=============================================================================
// k
//=============================================================================
//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct SmallVS_OUTPUT
{
    float4 Position : POSITION;
    float2 TexCoord0 : TEXCOORD0;
    float2 TexCoord1 : TEXCOORD1;
    float2 TexCoord2 : TEXCOORD2;
    float2 TexCoord3 : TEXCOORD3;
    float2 TexCoord4 : TEXCOORD4;
    float2 TexCoord5 : TEXCOORD5;
    float2 TexCoord6 : TEXCOORD6;
    float2 TexCoord7 : TEXCOORD7;
};


//-----------------------------------------------------------------------------
// Name: LogVS
//-----------------------------------------------------------------------------
SmallVS_OUTPUT SmallVS( float4 vPos : POSITION, 
                    float2 vTexCoord0 : TEXCOORD0 )
{
    SmallVS_OUTPUT Output;
  
    // tranform vertex position into screen space
    Output.Position = vPos;
    
    // Just copy the texture coordinate through
    Output.TexCoord0 = vTexCoord0 + float2( 0.5*g_fInvTexSize, 0.5*g_fInvTexSize );
    Output.TexCoord1 = vTexCoord0 + float2( 1.5*g_fInvTexSize, 0.5*g_fInvTexSize );
    Output.TexCoord2 = vTexCoord0 + float2( 3.5*g_fInvTexSize, 0.5*g_fInvTexSize );
    Output.TexCoord3 = vTexCoord0 + float2( 5.5*g_fInvTexSize, 0.5*g_fInvTexSize );
    Output.TexCoord4 = vTexCoord0 + float2( 0.5*g_fInvTexSize, 1.5*g_fInvTexSize );
    Output.TexCoord5 = vTexCoord0 + float2( 1.5*g_fInvTexSize, 1.5*g_fInvTexSize );
    Output.TexCoord6 = vTexCoord0 + float2( 3.5*g_fInvTexSize, 1.5*g_fInvTexSize );
    Output.TexCoord7 = vTexCoord0 + float2( 5.5*g_fInvTexSize, 1.5*g_fInvTexSize );
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: SmallPS
//-----------------------------------------------------------------------------
float4 SmallPS( SmallVS_OUTPUT In ) : COLOR
{
	float4 output;
	
    float4 t0 = tex2D( Sampler, In.TexCoord0 );
    float4 t1 = tex2D( Sampler, In.TexCoord1 );
    float4 t2 = tex2D( Sampler, In.TexCoord2 );
    float4 t3 = tex2D( Sampler, In.TexCoord3 );
    float4 t4 = tex2D( Sampler, In.TexCoord4 );
    float4 t5 = tex2D( Sampler, In.TexCoord5 );
    float4 t6 = tex2D( Sampler, In.TexCoord6 );
    float4 t7 = tex2D( Sampler, In.TexCoord7 );
    float4 t8 = tex2D( Sampler, In.TexCoord0 + float2(0,2.0*g_fInvTexSize) );
    float4 t9 = tex2D( Sampler, In.TexCoord1 + float2(0,2.0*g_fInvTexSize) );
    float4 ta = tex2D( Sampler, In.TexCoord2 + float2(0,2.0*g_fInvTexSize) );
    float4 tb = tex2D( Sampler, In.TexCoord3 + float2(0,2.0*g_fInvTexSize) );
    float4 tc = tex2D( Sampler, In.TexCoord4 + float2(0,2.0*g_fInvTexSize) );
    float4 td = tex2D( Sampler, In.TexCoord5 + float2(0,2.0*g_fInvTexSize) );
    float4 te = tex2D( Sampler, In.TexCoord6 + float2(0,2.0*g_fInvTexSize) );
    float4 tf = tex2D( Sampler, In.TexCoord7 + float2(0,2.0*g_fInvTexSize) );
    
    float l_max = max(max(max(max(t0.r,t1.r),max(t2.r,t3.r)),
						  max(max(t4.r,t5.r),max(t6.r,t7.r))),
					  max(max(max(t8.r,t9.r),max(ta.r,tb.r)),
					      max(max(tc.r,td.r),max(te.r,tf.r))));
    
	output.r = l_max;
	output.gba = (1.0/16.0)* (t0.gba + t1.gba + t2.gba + t3.gba
							+ t4.gba + t5.gba + t6.gba + t7.gba
							+ t8.gba + t9.gba + ta.gba + tb.gba
							+ tc.gba + td.gba + te.gba + tf.gba);
	
	return output;
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique SmallTechnique
{
    pass P0
    {
        VertexShader = compile vs_1_1 SmallVS();
        PixelShader  = compile ps_2_0 SmallPS();
    }
}





//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct FinalVS_OUTPUT
{
    float4 Position : POSITION;
    float2 TexCoord : TEXCOORD0;
};


//-----------------------------------------------------------------------------
// Name: FinalVS
//-----------------------------------------------------------------------------
FinalVS_OUTPUT FinalVS( float4 vPos : POSITION, 
                    float2 vTexCoord0 : TEXCOORD0 )
{
    FinalVS_OUTPUT Output;
  
    // tranform vertex position into screen space
    Output.Position = vPos - float4( 0.5*g_fInvTexSize, 0.5*g_fInvTexSize, 0, 0 );
    
    // Just copy the texture coordinate through
    Output.TexCoord = vTexCoord0;
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: FinalPS
//-----------------------------------------------------------------------------
float4 FinalPS( FinalVS_OUTPUT In ) : COLOR
{
	const float3 RGB2Y   = float3( +0.29900f, +0.58700f, +0.11400f );
	const float3 RGB2Cb  = float3( -0.16874f, -0.33126f, +0.50000f );
	const float3 RGB2Cr  = float3( +0.50000f, -0.41869f, -0.08131f );
	const float3 YCbCr2R = float3( +1.00000f, +0.00000f, +1.40200f );
	const float3 YCbCr2G = float3( +1.00000f, -0.34414f, -0.71414f );
	const float3 YCbCr2B = float3( +1.00000f, +1.77200f, +0.00000f );
	
	float4 info  = tex2D( SamplerInfo, float2(0.5,0.5) );
    float3 texel = tex2D( Sampler, In.TexCoord ).rgb;
	
	float coeff = 0.18 * exp( -info.g );
	float l_max = coeff * info.r;

	// YCbCrnɕϊ
	float3 YCbCr;
	YCbCr.y = dot( RGB2Cb, texel );
	YCbCr.z = dot( RGB2Cr, texel );

	// F͕̋␳
	float lum = coeff * dot( RGB2Y,  texel );
	YCbCr.x = lum * (1.0f+lum/(l_max*l_max)) / (1.0f+lum);
	
	// RGBnɂďo
	float4 color;
	color.r = dot( YCbCr2R,  YCbCr );
	color.g = dot( YCbCr2G,  YCbCr );
	color.b = dot( YCbCr2B,  YCbCr );
	color.a = 0;
	
	return color;
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique FinalTechnique
{
    pass P0
    {
        VertexShader = compile vs_1_1 FinalVS();
        PixelShader  = compile ps_2_0 FinalPS();
    }
}





